import torch
from torch import Tensor
from torch.linalg import vector_norm
from torch.nn import Module

from haplo.nicer_transform import PrecomputedUnnormalizePhaseAmplitudes


class PlusOneChiSquaredStatisticMetric(Module):
    def forward(self, output: torch.Tensor, target: torch.Tensor):
        unnormalize_phase_amplitudes = PrecomputedUnnormalizePhaseAmplitudes()
        observed = unnormalize_phase_amplitudes(output.type(torch.float64)) + 1.0
        expected = unnormalize_phase_amplitudes(target.type(torch.float64)) + 1.0
        chi_squared_statistic_f64 = torch.mean(torch.sum(((observed - expected) ** 2) / expected, dim=1))
        chi_squared_statistic = chi_squared_statistic_f64.type(torch.float32)
        return chi_squared_statistic


class PlusOneBeforeUnnormalizationChiSquaredStatisticMetric(Module):
    def forward(self, output: torch.Tensor, target: torch.Tensor):
        unnormalize_phase_amplitudes = PrecomputedUnnormalizePhaseAmplitudes()
        observed = unnormalize_phase_amplitudes(output.type(torch.float64) + 1.0)
        expected = unnormalize_phase_amplitudes(target.type(torch.float64) + 1.0)
        chi_squared_statistic_f64 = torch.mean(torch.sum(((observed - expected) ** 2) / expected, dim=1))
        chi_squared_statistic = chi_squared_statistic_f64.type(torch.float32)
        return chi_squared_statistic


class SumDifferenceSquaredOverMedianExpectedSquaredMetric(Module):
    def forward(self, output: torch.Tensor, target: torch.Tensor):
        epsilon = 1e-10
        unnormalize_phase_amplitudes = PrecomputedUnnormalizePhaseAmplitudes()
        observed = unnormalize_phase_amplitudes(output.type(torch.float64)) + 1.0
        expected = unnormalize_phase_amplitudes(target.type(torch.float64)) + 1.0
        numerator = torch.sum(((observed - expected) ** 2), dim=1)
        median = torch.median(expected, dim=1).values
        denominator = median ** 2
        quality_indicator = numerator / denominator
        metric_f64 = torch.mean(torch.log10(quality_indicator + epsilon))
        metric = metric_f64.type(torch.float32)
        return metric

def norm_based_gradient_clip(gradient_tensor: Tensor) -> Tensor:
    return gradient_tensor / torch.maximum(torch.linalg.vector_norm(gradient_tensor), torch.tensor(1.0))



from __future__ import annotations

import itertools
import logging
import math
from pathlib import Path
from typing import Optional, Callable, List

import numpy as np
import pandas as pd
import polars as pl
import torch
from sqlalchemy import create_engine, text
from sqlalchemy.orm import sessionmaker
from torch.utils.data import Dataset, Subset, get_worker_info

logger = logging.getLogger(__name__)


class NicerDataset(Dataset):
    def __init__(self, database_uri: str, database_path: Path, length: int, parameter_count: int,
                 parameters_transform: Optional[Callable] = None, phase_amplitudes_transform: Optional[Callable] = None,
                 *, in_memory: bool = False):
        self.database_uri: str = database_uri
        self.database_path: Path = database_path
        self.parameters_transform: Callable = parameters_transform
        self.phase_amplitudes_transform: Callable = phase_amplitudes_transform
        # TODO: Quick hack. Should not being doing logic in init. Move this to factory method.
        self.length: int = length
        self.engine = None
        self.connection = None
        self.in_memory = in_memory
        self.shared_data_frame = None
        self.parameter_count: int = parameter_count

    @classmethod
    def new(cls, dataset_path: Path, parameters_transform: Optional[Callable] = None,
            phase_amplitudes_transform: Optional[Callable] = None, in_memory: bool = False,
            length: Optional[int] = None, parameter_count: int = 11):
        database_uri = f'sqlite:///{dataset_path}?mode=ro'
        database_path = dataset_path
        if length is None:
            engine = create_engine(database_uri)
            connection = engine.connect()
            count_data_frame = pl.read_database(query='select count(1) from main', connection=connection)
            count_row = count_data_frame.row(0)
            length = count_row[0]
        instance = cls(database_uri=database_uri, database_path=database_path, length=length,
                       parameter_count=parameter_count, parameters_transform=parameters_transform,
                       phase_amplitudes_transform=phase_amplitudes_transform, in_memory=in_memory)
        return instance

    def __len__(self):
        return self.length

    def __getitem__(self, index):
        initialize_connection(self)  # TODO: Probably shouldn't be necessary.
        row_index = index + 1  # The SQL database auto increments from 1, not 0.
        row = self.get_row_from_index(row_index)
        parameters = np.array(row[-self.parameter_count-64:-64], dtype=np.float32)
        phase_amplitudes = np.array(row[-64:], dtype=np.float32)
        if self.parameters_transform is not None:
            parameters = self.parameters_transform(parameters)
        if self.phase_amplitudes_transform is not None:
            phase_amplitudes = self.phase_amplitudes_transform(phase_amplitudes)
        return parameters, phase_amplitudes

    def get_row_from_index(self, row_index):
        # logger.info(f'Accessing rowid {row_index} from {self.database_uri}.')
        row_data_frame = pl.read_database(query=rf'select * from main where ROWID = {row_index}',
                                          connection=self.connection)
        row = row_data_frame.row(0)
        return row

    def get_row_from_in_memory(self, row_index):
        row_data_frame = self.shared_data_frame.loc[row_index]
        row = row_data_frame.values
        return row

    def get_rows_from_indexes_with_row_id_column(self, row_indexes) -> pd.DataFrame:
        initialize_connection(self)
        row_indexes = list(np.asarray(row_indexes) + 1)
        indexes_sql_string = ', '.join(map(str, row_indexes))
        data_frame = pl.read_database(query=rf'select ROWID, * from main where ROWID in ({indexes_sql_string})',
                                      connection=self.connection)
        pandas_data_frame = data_frame.to_pandas()
        pandas_data_frame.set_index('rowid', inplace=True)
        return pandas_data_frame


def nicer_dataset_worker_initialization_function(worker_id: int) -> None:
    # TODO: Hacked way of getting dataset.
    worker_info = get_worker_info()
    dataset: NicerDataset = worker_info.dataset.dataset
    initialize_connection(dataset)


def initialize_connection(dataset: NicerDataset):
    if dataset.engine is None:
        dataset.engine = create_engine(dataset.database_uri)
        dataset.connection = dataset.engine.connect()


def disconnect(dataset: NicerDataset):
    dataset.engine = None
    dataset.connection = None


def split_into_train_validation_and_test_datasets(dataset: NicerDataset) -> (NicerDataset, NicerDataset, NicerDataset):
    length_10_percent = round(len(dataset) * 0.1)
    train_dataset = Subset(dataset, range(length_10_percent * 8))
    validation_dataset = Subset(dataset, range(length_10_percent * 8, length_10_percent * 9))
    test_dataset = Subset(dataset, range(length_10_percent * 9, len(dataset)))
    return train_dataset, validation_dataset, test_dataset


def split_dataset_into_fractional_datasets(dataset: NicerDataset, fractions: List[float]) -> List[NicerDataset]:
    assert np.isclose(np.sum(fractions), 1.0)
    fractional_datasets: List[NicerDataset] = []
    cumulative_fraction = 0
    previous_index = 0
    for fraction in fractions:
        cumulative_fraction += fraction
        if np.isclose(cumulative_fraction, 1.0):
            next_index = len(dataset)
        else:
            next_index = round(len(dataset) * cumulative_fraction)
        indexes = torch.tensor(range(previous_index, next_index), dtype=torch.int32)
        fractional_dataset: NicerDataset = Subset(dataset, indexes)
        fractional_datasets.append(fractional_dataset)
        previous_index = next_index
    return fractional_datasets


def split_dataset_into_count_datasets(dataset: NicerDataset, counts: List[int]) -> List[NicerDataset]:
    assert np.sum(counts) < len(dataset)
    count_datasets: List[NicerDataset] = []
    next_index = 0
    previous_index = 0
    for count in counts:
        next_index += count
        indexes = torch.tensor(range(previous_index, next_index), dtype=torch.int32)
        count_dataset: NicerDataset = Subset(dataset, indexes)
        count_datasets.append(count_dataset)
        previous_index = next_index
    indexes = torch.tensor(range(previous_index, len(dataset)), dtype=torch.int32)
    count_datasets.append(Subset(dataset, indexes))
    return count_datasets


def move_sqlite_subset_to_new_file(source_database_path: Path, target_database_path: Path, row_indexes: List[int]):
    source_database_uri = f'sqlite:///{source_database_path}?mode=ro'
    target_database_uri = f'sqlite:///{target_database_path}'
    row_ids = list(np.asarray(row_indexes) + 1)
    ids_sql_string = ', '.join(map(str, row_ids))
    source_engine = create_engine(source_database_uri)
    source_connection = source_engine.connect()
    data_frame = pl.read_database(query=rf'select ROWID, * from main where ROWID in ({ids_sql_string})',
                                  connection=source_connection)
    row_count = data_frame.select(pl.count()).item()
    logger.info(f'Loaded {row_count} rows.')
    source_connection.close()
    data_frame.rename({'rowid': 'ROWID'})
    data_frame.write_database(table_name='main', connection=target_database_uri, if_table_exists='append')
    target_engine = create_engine(target_database_uri)
    Session = sessionmaker(bind=target_engine)
    session = Session()
    logger.info(f'Creating index for local file...')
    session.execute(text('CREATE INDEX index_rowid ON main(rowid);'))
    session.close()



from pathlib import Path

from haplo.nicer_dataset import NicerDataset


def main():
    dataset_path = Path('data_nobackup/640m_parameters_and_phase_amplitudes.db')
    dataset = NicerDataset.new(dataset_path=dataset_path)

    example0 = dataset[0]
    parameters0, phase_amplitudes0 = example0
    print(parameters0)
    print(phase_amplitudes0)


if __name__ == '__main__':
    main()



import inspect
import os
import subprocess
from pathlib import Path

from torch.distributed import init_process_group

from haplo.train_system_configuration import TrainSystemConfiguration


def ddp_setup(system_configuration: TrainSystemConfiguration):
    distributed_back_end = system_configuration.distributed_back_end
    if 'RANK' not in os.environ:
        # The script was not called with `torchrun` and environment variables need to be set manually.
        os.environ['RANK'] = str(0)
        os.environ['LOCAL_RANK'] = str(0)
        os.environ['WORLD_SIZE'] = str(1)
        os.environ['LOCAL_WORLD_SIZE'] = str(1)
        os.environ["MASTER_ADDR"] = "localhost"
        os.environ["MASTER_PORT"] = "35728"
    init_process_group(backend=distributed_back_end)


def distributed_logging(decorated_function):
    if 'HAPLO_DISTRIBUTED_LOGGING_ENABLED' in os.environ:
        return decorated_function
    if 'RANK' not in os.environ:
        return decorated_function
    if 'HAPLO_SESSION_DIRECTORY' not in os.environ:
        return decorated_function
    else:
        def function_in_subprocess():
            decorated_file = inspect.getfile(decorated_function)
            subprocess_environment = os.environ.copy()
            subprocess_environment['HAPLO_DISTRIBUTED_LOGGING_ENABLED'] = '1'
            session_directory = Path(os.environ['HAPLO_SESSION_DIRECTORY'])
            session_directory.mkdir(parents=True, exist_ok=True)
            with session_directory.joinpath(f'rank_{os.environ["RANK"]}_group_rank_{os.environ["GROUP_RANK"]}.log'
                                            ).open('a') as output_file:
                subprocess.run(['python', decorated_file], env=subprocess_environment,
                               stdout=output_file, stderr=subprocess.STDOUT)

        return function_in_subprocess



import math
import os
from pathlib import Path

import torch
from haplo.data_paths import move_to_tmp_on_pbs
from haplo.distributed import distributed_logging
from haplo.unwrap_model import unwrap_model
from torch.optim import AdamW
from datetime import timedelta
from haplo.losses import PlusOneBeforeUnnormalizationChiSquaredStatisticMetric, \
    PlusOneChiSquaredStatisticMetric, SumDifferenceSquaredOverMedianExpectedSquaredMetric
from haplo.models import Cura
from haplo.nicer_dataset import NicerDataset, split_dataset_into_count_datasets
from haplo.train_hyperparameter_configuration import TrainHyperparameterConfiguration
from haplo.train_logging_configuration import TrainLoggingConfiguration
from haplo.train_system_configuration import TrainSystemConfiguration
from haplo.train_session import train_session
from haplo.nicer_transform import PrecomputedNormalizeParameters, PrecomputedNormalizePhaseAmplitudes
from haplo.pbs_helper import schedule_self_process_interrupt_signal_before_pbs_end_time
from haplo.train_session import load_latest_state
 

@distributed_logging
def example_train_session():
    full_dataset_path = Path('data/640m_parameters_and_phase_amplitudes.db')
    full_train_dataset = NicerDataset.new(
        dataset_path=full_dataset_path,
        length=600_000_000,
        parameters_transform=PrecomputedNormalizeParameters(),
        phase_amplitudes_transform=PrecomputedNormalizePhaseAmplitudes(),
        in_memory= True
    )
    test_dataset, validation_dataset, train_dataset, _ = split_dataset_into_count_datasets(
        full_train_dataset, [100_000, 100_000, 50_000_000])
    model = Cura()

    loss_function = SumDifferenceSquaredOverMedianExpectedSquaredMetric()
    metric_functions = [PlusOneChiSquaredStatisticMetric(), PlusOneBeforeUnnormalizationChiSquaredStatisticMetric(),SumDifferenceSquaredOverMedianExpectedSquaredMetric()]
    hyperparameter_configuration = TrainHyperparameterConfiguration.new(batch_size=1000)
    system_configuration = TrainSystemConfiguration.new(preprocessing_processes_per_train_process=2)
    optimizer = AdamW(params=model.parameters(), lr=hyperparameter_configuration.learning_rate,
                      weight_decay=hyperparameter_configuration.weight_decay,
                      eps=hyperparameter_configuration.optimizer_epsilon)

    #load_latest_state(model=model, optimizer=optimizer, session_directory=Path('sessions/2024_02_20_08_34_17_50m_continue_diff_loss_multinode_in_mem_2pp_retry'))

    run_comments = (f'TEST RUN')  # Whatever you want to log in a string.
    additional_log_dictionary = {
        'model_name': type(model).__name__, 'train_dataset_size': len(train_dataset), 'run_comments': run_comments,
        'pbs_job_id': os.environ['PBS_JOBID']
    }
    schedule_self_process_interrupt_signal_before_pbs_end_time(timedelta(minutes=2))
    logging_configuration = TrainLoggingConfiguration.new(
        wandb_project='multinode_runs', wandb_entity='ramjet', additional_log_dictionary=additional_log_dictionary,
        model_save_cycle_frequency=math.ceil(500_000_000 / len(train_dataset))
    )
    train_session(train_dataset=train_dataset, validation_dataset=validation_dataset, model=model,
                  loss_function=loss_function, metric_functions=metric_functions, optimizer=optimizer,
                  hyperparameter_configuration=hyperparameter_configuration, system_configuration=system_configuration,
                  logging_configuration=logging_configuration)


if __name__ == '__main__':
    example_train_session()



from collections import OrderedDict


def unwrap_model(model_dict):
    unwrapped_model_dict = OrderedDict()
    for key, value in model_dict.items():
        if key.startswith('module.'):
            unwrapped_key = key[7:]
        else:
            unwrapped_key = key
        unwrapped_model_dict[unwrapped_key] = value
    return unwrapped_model_dict



import torch

from haplo.losses import norm_based_gradient_clip


def test_norm_based_gradient_clipping():
    gradient_tensor = torch.tensor([3, 4], dtype=torch.float32)
    # Simple 3**2 + 4**2 = 5**2 should result in the values divided by 5.
    expected_normalized_gradient_tensor = torch.tensor([3/5, 4/5], dtype=torch.float32)
    normalized_gradient_tensor = norm_based_gradient_clip(gradient_tensor)
    assert torch.allclose(normalized_gradient_tensor, expected_normalized_gradient_tensor)


def test_norm_based_gradient_clipping_when_norm_limit_is_not_reached():
    gradient_tensor = torch.tensor([3/6, 4/6], dtype=torch.float32)
    # Since the norm of 3 and 4 is 5, dividing by 6 puts the norm below 1.
    expected_normalized_gradient_tensor = torch.tensor([3/6, 4/6], dtype=torch.float32)
    normalized_gradient_tensor = norm_based_gradient_clip(gradient_tensor)
    assert torch.allclose(normalized_gradient_tensor, expected_normalized_gradient_tensor)



from datetime import timedelta

from dataclasses import dataclass

from torch.distributed import Backend


@dataclass
class TrainSystemConfiguration:
    """
    Configuration settings for the system of a train session.

    :ivar preprocessing_processes_per_train_process: The number of processes that are started to preprocess the data
        per train process. The train session will create this many processes for each the train data and the validation
        data.
    """
    preprocessing_processes_per_train_process: int
    distributed_back_end: Backend
    data_loader_creation_barrier_timeout: timedelta

    @classmethod
    def new(cls,
            preprocessing_processes_per_train_process: int = 10,
            distributed_back_end: Backend = Backend.GLOO,
            data_loader_creation_barrier_timeout: timedelta = timedelta(hours=1.0)):
        return cls(preprocessing_processes_per_train_process=preprocessing_processes_per_train_process,
                   distributed_back_end=distributed_back_end,
                   data_loader_creation_barrier_timeout=data_loader_creation_barrier_timeout)



from pathlib import Path

from bokeh.io import show
from bokeh.models import Column
from bokeh.plotting import figure

from haplo.nicer_dataset import NicerDataset


def example_data_visualization():
    dataset_path = Path('data/800k_parameters_and_phase_amplitudes.db')
    dataset = NicerDataset.new(dataset_path=dataset_path)
    
    example0 = dataset[0]
    parameters0, phase_amplitudes0 = example0

    parameters_figure = figure(x_axis_label='parameter', y_axis_label='value')
    parameters_figure.vbar(x=list(range(11)), top=parameters0)

    phase_amplitudes_figure = figure(x_axis_label='phase', y_axis_label='amplitude')
    phase_amplitudes_figure.line(x=list(range(64)), y=phase_amplitudes0)
    
    column = Column(parameters_figure, phase_amplitudes_figure)
    show(column)


if __name__ == '__main__':
    example_data_visualization()


import logging
from pathlib import Path

import torch
from bokeh.io import show
from bokeh.models import Column
from bokeh.plotting import figure as Figure
from torch import Tensor
from torch.nn import Module, DataParallel
from torch.utils.data import DataLoader

from haplo.logging import set_up_default_logger
from haplo.losses import PlusOneChiSquaredStatisticMetric
from haplo.models import Cura
from haplo.nicer_dataset import NicerDataset, split_dataset_into_fractional_datasets
from haplo.nicer_transform import PrecomputedNormalizeParameters, PrecomputedNormalizePhaseAmplitudes

logger = logging.getLogger(__name__)


def infer_session(dataset_path: Path):
    set_up_default_logger()
    if torch.cuda.is_available():
        device = torch.device('cuda')
    else:
        device = torch.device('cpu')

    evaluation_dataset = NicerDataset.new(
        dataset_path=dataset_path,
        parameters_transform=PrecomputedNormalizeParameters(),
        phase_amplitudes_transform=PrecomputedNormalizePhaseAmplitudes())
    validation_dataset, test_dataset = split_dataset_into_fractional_datasets(evaluation_dataset, [0.5, 0.5])

    batch_size = 100

    test_dataloader = DataLoader(test_dataset, batch_size=batch_size)

    model = DataParallel(Cura())
    model = model.to(device)
    model.load_state_dict(torch.load('sessions/rural-totem-623_lowest_validation_model.pt', map_location=device))
    model.eval()
    loss_function = PlusOneChiSquaredStatisticMetric()

    phase_test(test_dataloader, model, loss_function, device=device)


def phase_test(dataloader, model_: Module, loss_fn, device):
    num_batches = len(dataloader)
    test_loss, correct = 0, 0

    with torch.no_grad():
        for batch_index, (inputs_tensor, targets) in enumerate(dataloader):
            logger.info(f'Processed {batch_index * dataloader.batch_size} examples.')
            inputs_tensor = inputs_tensor.to(device)
            targets = targets
            predicted_targets = model_(inputs_tensor)
            test_loss += loss_fn(predicted_targets.to('cpu'), targets).to(device).item()
            pass

    test_loss /= num_batches
    logger.info(f"Test Error: \nAvg loss: {test_loss:>8f} \n")


def quick_view(predicted_light_curve: Tensor, light_curve: Tensor) -> Figure:
    figure = Figure()
    predicted_light_curve_array = predicted_light_curve.to('cpu').numpy()
    light_curve_array = light_curve.to('cpu').numpy()
    figure.line(x=list(range(len(predicted_light_curve_array))), y=predicted_light_curve_array, line_color='firebrick')
    figure.line(x=list(range(len(light_curve_array))), y=light_curve_array, line_color='mediumblue')
    return figure


def show_all(x_list, y_list):
    column_contents = []
    for x, y in zip(x_list, y_list):
        column_contents.append(quick_view(x, y))
    column = Column(*column_contents)
    show(column)




if __name__ == '__main__':
    dataset_path = Path('data/800k_parameters_and_phase_amplitudes.db')
    infer_session(dataset_path)



import numpy as np
import pytest

from haplo.nicer_transform import PrecomputedNormalizePhaseAmplitudes, PrecomputedUnnormalizePhaseAmplitudes, \
    PrecomputedUnnormalizeParameters, PrecomputedNormalizeParameters


def test_precomputed_normalize_phase_amplitudes_values():
    x = np.ones([1, 64], dtype=np.float32)
    expected_y = np.full(shape=[1, 64], fill_value=-0.71331304)
    transform = PrecomputedNormalizePhaseAmplitudes()
    y = transform(x)
    assert np.allclose(y, expected_y)


@pytest.mark.skip(reason='Need to refactor to make this test work.')
def test_precomputed_normalize_phase_amplitudes_does_not_mutate_original_array():
    x = np.ones([1, 64], dtype=np.float32)
    transform = PrecomputedNormalizePhaseAmplitudes()
    _ = transform(x)
    assert np.allclose(x, np.ones([1, 64], dtype=np.float32))


def test_precomputed_unnormalize_phase_amplitudes_values():
    x = np.ones([1, 64], dtype=np.float32)
    expected_y = np.full(shape=[1, 64], fill_value=81723.75)
    transform = PrecomputedUnnormalizePhaseAmplitudes()
    y = transform(x)
    assert np.allclose(y, expected_y)


@pytest.mark.skip(reason='Need to refactor to make this test work.')
def test_precomputed_unnormalize_phase_amplitudes_does_not_mutate_original_array():
    x = np.ones([1, 64], dtype=np.float32)
    transform = PrecomputedUnnormalizePhaseAmplitudes()
    _ = transform(x)
    assert np.allclose(x, np.ones([1, 64], dtype=np.float32))


def test_precomputed_normalize_parameters_values():
    x = np.ones([1, 11], dtype=np.float32)
    expected_y = np.array([[3.5573754, 3.5618417, 3.553724, -0.6303051, -1.1805683, 3.556168, 3.5511322, 3.557625,
                            -0.63149905, -1.1832114, -1.5620476]])
    transform = PrecomputedNormalizeParameters()
    y = transform(x)
    assert np.allclose(y, expected_y)


@pytest.mark.skip(reason='Need to refactor to make this test work.')
def test_precomputed_normalize_parameters_does_not_mutate_original_array():
    x = np.ones([1, 11], dtype=np.float32)
    transform = PrecomputedNormalizeParameters()
    _ = transform(x)
    assert np.allclose(x, np.ones([1, 11], dtype=np.float32))


def test_precomputed_unnormalize_parameters_values():
    x = np.ones([1, 11], dtype=np.float32)
    expected_y = np.array([[0.2805303, 0.28011018, 0.28137863, 2.478689, 4.950499, 0.28018776, 0.28149468, 0.28105912,
                            2.4785657, 4.960278, 8.396504]])
    transform = PrecomputedUnnormalizeParameters()
    y = transform(x)
    assert np.allclose(y, expected_y)


@pytest.mark.skip(reason='Need to refactor to make this test work.')
def test_precomputed_unnormalize_parameters_does_not_mutate_original_array():
    x = np.ones([1, 11], dtype=np.float32)
    transform = PrecomputedUnnormalizeParameters()
    _ = transform(x)
    assert np.allclose(x, np.ones([1, 11], dtype=np.float32))



from pathlib import Path

import numpy as np
import torch
from bokeh.io import show
from bokeh.plotting import figure

from haplo.export_onnx import WrappedModel
from haplo.models import Cura
from haplo.nicer_dataset import NicerDataset, split_dataset_into_count_datasets
from haplo.nicer_transform import PrecomputedNormalizeParameters, PrecomputedNormalizePhaseAmplitudes, \
    PrecomputedUnnormalizePhaseAmplitudes


def example_infer_session():
    model = Cura()
    model = WrappedModel(model)  # The DDP module requires an extra wrapping. This emulates that.
    saved_model_path = Path('/Users/golmschenk/Downloads/lowest_50m_validation_model.pt')
    model.load_state_dict(torch.load(str(saved_model_path), map_location=torch.device('cpu')))
    model.eval()
    normalize = PrecomputedNormalizeParameters()
    unnormalize = PrecomputedUnnormalizePhaseAmplitudes()
    parameters = np.array([7.557257540694740E-002, 0.123674255335847, -0.494945657705134,
                           2.16487304204509, 2.45025873920033, 0.338851445495323,
                           -0.126617284613881, -9.594016785496662E-002, 1.76792362926050,
                           5.41682185280249, 4.27309470936332], dtype=np.float32)
    normalized_parameters = normalize(parameters)
    input_array = np.expand_dims(normalized_parameters, axis=0)
    with torch.no_grad():
        input_tensor = torch.tensor(input_array)
        output_tensor = model(input_tensor)
        output_array = output_tensor.numpy()
    normalized_phase_amplitudes = np.squeeze(output_array, axis=0)
    phase_amplitudes = unnormalize(normalized_phase_amplitudes)
    print(phase_amplitudes)
    for phase_amplitude in phase_amplitudes:
        print(f'{phase_amplitude}, ', end='')
    pass


if __name__ == '__main__':
    example_infer_session()



from pathlib import Path

import pandas as pd
import sqlite3

from haplo.nicer_dataset import NicerDataset, split_dataset_into_count_datasets, split_dataset_into_fractional_datasets

# TODO: The test datasets should be in memory, not saved to disk.
database_path = Path(__file__ + '.temp.db')


def create_fake_data(rows: int = 3):
    database_path.unlink(missing_ok=True)
    database_path.parent.mkdir(exist_ok=True, parents=True)
    connection = sqlite3.connect(database_path)
    fake_data = []
    parameter_count = 11
    phase_amplitude_count = 64
    parameter_column_names = [f'parameter{index}' for index in range(parameter_count)]
    phase_amplitude_column_names = [f'phase_amplitude{index}' for index in range(phase_amplitude_count)]
    data_column_names = parameter_column_names + phase_amplitude_column_names
    for index in range(rows):
        fake_data.append({name: name_index + (64 * index) for name_index, name in enumerate(data_column_names)})
    data_frame = pd.DataFrame(fake_data)
    data_frame.to_sql(name='main', con=connection, index=False)


def test_getitem():
    create_fake_data()
    dataset = NicerDataset.new(database_path)
    parameters1, phase_amplitudes1 = dataset[1]
    assert parameters1[3] == 67
    assert phase_amplitudes1[3] == 78


def test_len():
    create_fake_data()
    dataset = NicerDataset.new(database_path)
    assert len(dataset) == 3


def test_len_after_factional_split():
    create_fake_data(rows=8)
    full_dataset = NicerDataset.new(database_path)
    fractional_dataset0, fractional_dataset1 = split_dataset_into_fractional_datasets(full_dataset, [0.25, 0.75])
    assert len(fractional_dataset0) == 2
    assert len(fractional_dataset1) == 6


def test_len_after_count_split():
    create_fake_data(rows=8)
    full_dataset = NicerDataset.new(database_path)
    count_dataset0, count_dataset1, count_dataset2 = split_dataset_into_count_datasets(full_dataset, [2, 5])
    assert len(count_dataset0) == 2
    assert len(count_dataset1) == 5
    assert len(count_dataset2) == 1



from pathlib import Path

import torch

from haplo.export_onnx import WrappedModel, export_onnx
from haplo.models import Cura


def example_export_to_onnx():
    pytorch_path = Path('sessions/your_model_name.pt')
    model = Cura()
    model = WrappedModel(model)
    model.load_state_dict(torch.load(pytorch_path, map_location='cpu'))
    export_onnx(model=model, output_onnx_model_path=Path('exported_model.onnx'))


if __name__ == '__main__':
    example_export_to_onnx()
